"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const path = require('path');
const seq = require('promisequence');
const chalk = require('chalk');
const createDebug = require('debug');
const got_1 = require('../network/got');
const initCmd_1 = require('./initCmd');
const installMultiple_1 = require('../installMultiple');
const save_1 = require('../save');
const linkPeers_1 = require('../install/linkPeers');
const runtimeError_1 = require('../runtimeError');
const getSaveType_1 = require('../getSaveType');
const runScript_1 = require('../runScript');
const postInstall_1 = require('../install/postInstall');
const linkBins_1 = require('../install/linkBins');
const defaults_1 = require('../defaults');
const pnpmPkgJson_1 = require('../pnpmPkgJson');
/*
 * Perform installation.
 *
 *     install({'lodash': '1.0.0', 'foo': '^2.1.0' }, { quiet: true })
 */
function default_1(fuzzyDeps, optsNullable) {
    return __awaiter(this, void 0, void 0, function* () {
        let packagesToInstall = mapify(fuzzyDeps);
        const installType = packagesToInstall && Object.keys(packagesToInstall).length ? 'named' : 'general';
        const opts = Object.assign({}, defaults_1.default, optsNullable);
        const isProductionInstall = opts.production || process.env.NODE_ENV === 'production';
        const baseCmd = yield initCmd_1.default(opts);
        const cmd = Object.assign(baseCmd, {
            ctx: Object.assign({}, baseCmd.ctx, {
                fetches: {},
                builds: {},
                installs: {},
                got: got_1.default({
                    concurrency: opts.concurrency,
                    fetchRetries: opts.fetchRetries,
                    fetchRetryFactor: opts.fetchRetryFactor,
                    fetchRetryMintimeout: opts.fetchRetryMintimeout,
                    fetchRetryMaxtimeout: opts.fetchRetryMaxtimeout
                })
            })
        });
        try {
            if (installType !== 'named') {
                if (!cmd.pkg || !cmd.pkg.pkg)
                    throw runtimeError_1.default('No package.json found');
                packagesToInstall = Object.assign({}, cmd.pkg.pkg.dependencies || {});
                if (!isProductionInstall)
                    Object.assign(packagesToInstall, cmd.pkg.pkg.devDependencies || {});
            }
            const pkgs = yield installMultiple_1.default(cmd.ctx, packagesToInstall, cmd.pkg && cmd.pkg.pkg && cmd.pkg.pkg.optionalDependencies || {}, path.join(cmd.ctx.root, 'node_modules'), Object.assign({}, opts, { dependent: cmd.pkg && cmd.pkg.path || cmd.ctx.root }));
            if (installType === 'named') {
                const saveType = getSaveType_1.default(opts);
                if (saveType) {
                    if (!cmd.pkg) {
                        throw new Error('Cannot save because no package.json found');
                    }
                    const inputNames = Object.keys(packagesToInstall);
                    const savedPackages = pkgs.filter((pkg) => inputNames.indexOf(pkg.pkg.name) > -1);
                    yield save_1.default(cmd.pkg.path, savedPackages, saveType, opts.saveExact);
                }
            }
            cmd.storeJsonCtrl.save(Object.assign(cmd.ctx.storeJson, {
                pnpm: pnpmPkgJson_1.default.version
            }));
            yield linkPeers_1.default(cmd.ctx.store, cmd.ctx.installs);
            // postinstall hooks
            if (!(opts.ignoreScripts || !cmd.ctx.piq || !cmd.ctx.piq.length)) {
                yield seq(cmd.ctx.piq.map(pkg => () => linkBins_1.default(path.join(pkg.path, '_', 'node_modules'))
                    .then(() => postInstall_1.default(pkg.path, installLogger(pkg.pkgFullname)))
                    .catch(err => {
                    if (cmd.ctx.installs[pkg.pkgFullname].optional) {
                        console.log('Skipping failed optional dependency ' + pkg.pkgFullname + ':');
                        console.log(err.message || err);
                        return;
                    }
                    throw err;
                })));
            }
            yield linkBins_1.default(path.join(cmd.ctx.root, 'node_modules'));
            if (!opts.ignoreScripts && cmd.pkg) {
                yield mainPostInstall(cmd.pkg.pkg && cmd.pkg.pkg.scripts || {}, cmd.ctx.root, isProductionInstall);
            }
            yield cmd.unlock();
        }
        catch (err) {
            if (cmd && cmd.unlock)
                cmd.unlock();
            throw err;
        }
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
function mainPostInstall(scripts, pkgRoot, isProductionInstall) {
    if (scripts['postinstall'])
        npmRun('postinstall', pkgRoot);
    if (!isProductionInstall && scripts['prepublish'])
        npmRun('prepublish', pkgRoot);
}
function npmRun(scriptName, pkgRoot) {
    const result = runScript_1.sync('npm', ['run', scriptName], {
        cwd: pkgRoot,
        stdio: 'inherit'
    });
    if (result.status !== 0) {
        process.exit(result.status);
    }
}
function installLogger(pkgFullname) {
    return (stream, line) => {
        createDebug('pnpm:post_install')(`${pkgFullname} ${line}`);
        if (stream === 'stderr') {
            console.log(chalk.blue(pkgFullname) + '! ' + chalk.gray(line));
            return;
        }
        console.log(chalk.blue(pkgFullname) + '  ' + chalk.gray(line));
    };
}
function mapify(pkgs) {
    if (!pkgs)
        return {};
    if (Array.isArray(pkgs)) {
        return pkgs.reduce((pkgsMap, pkgFullName) => {
            const matches = /(@?[^@]+)@(.*)/.exec(pkgFullName);
            if (!matches) {
                pkgsMap[pkgFullName] = '*';
            }
            else {
                pkgsMap[matches[1]] = matches[2];
            }
            return pkgsMap;
        }, {});
    }
    return pkgs;
}
//# sourceMappingURL=install.js.map