"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const cbRimraf = require('rimraf');
const path = require('path');
const initCmd_1 = require('./initCmd');
const getSaveType_1 = require('../getSaveType');
const removeDeps_1 = require('../removeDeps');
const binify_1 = require('../binify');
const defaults_1 = require('../defaults');
const requireJson_1 = require('../fs/requireJson');
function uninstallCmd(pkgsToUninstall, optsNullable) {
    return __awaiter(this, void 0, void 0, function* () {
        const opts = Object.assign({}, defaults_1.default, optsNullable);
        const saveType = getSaveType_1.default(opts);
        const cmd = yield initCmd_1.default(opts);
        try {
            if (!cmd.pkg) {
                throw new Error('No package.json found - cannot uninstall');
            }
            const pkg = cmd.pkg; // NOTE: otherwise TypeScript thinks cmd.pkg might be undefined for some reason
            cmd.pkg.pkg.dependencies = cmd.pkg.pkg.dependencies || {};
            // this is OK. The store might not have records for the package
            // maybe it was cloned, `pnpm install` was not executed
            // and remove is done on a package with no dependencies installed
            cmd.ctx.storeJson.dependencies[cmd.pkg.path] = cmd.ctx.storeJson.dependencies[cmd.pkg.path] || [];
            const pkgFullNames = pkgsToUninstall
                .map(dep => cmd.ctx.storeJson.dependencies[pkg.path].find(_ => _.indexOf(`${dep}@`) === 0))
                .filter(pkgFullName => !!pkgFullName);
            const uninstalledPkgs = tryUninstall(pkgFullNames.slice(), cmd.ctx.storeJson, cmd.pkg.path);
            uninstalledPkgs.forEach(uninstalledPkg => removeBins(uninstalledPkg, cmd.ctx.store, cmd.ctx.root));
            if (cmd.ctx.storeJson.dependencies[cmd.pkg.path]) {
                pkgFullNames.forEach(dep => {
                    cmd.ctx.storeJson.dependencies[pkg.path].splice(cmd.ctx.storeJson.dependencies[pkg.path].indexOf(dep), 1);
                });
                if (!cmd.ctx.storeJson.dependencies[cmd.pkg.path].length) {
                    delete cmd.ctx.storeJson.dependencies[cmd.pkg.path];
                }
            }
            yield Promise.all(uninstalledPkgs.map(pkgFullName => removePkgFromStore(pkgFullName, cmd.ctx.store)));
            cmd.storeJsonCtrl.save(cmd.ctx.storeJson);
            yield Promise.all(pkgsToUninstall.map(dep => rimraf(path.join(cmd.ctx.root, 'node_modules', dep))));
            if (saveType) {
                yield removeDeps_1.default(cmd.pkg.path, pkgsToUninstall, saveType);
            }
            yield cmd.unlock();
        }
        catch (err) {
            if (typeof cmd !== 'undefined' && cmd.unlock)
                yield cmd.unlock();
            throw err;
        }
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = uninstallCmd;
function canBeUninstalled(pkgFullName, storeJson, pkgPath) {
    return !storeJson.dependents[pkgFullName] || !storeJson.dependents[pkgFullName].length ||
        storeJson.dependents[pkgFullName].length === 1 && storeJson.dependents[pkgFullName].indexOf(pkgPath) !== -1;
}
function tryUninstall(pkgFullNames, storeJson, pkgPath) {
    const uninstalledPkgs = [];
    let numberOfUninstalls;
    do {
        numberOfUninstalls = 0;
        for (let i = 0; i < pkgFullNames.length;) {
            if (canBeUninstalled(pkgFullNames[i], storeJson, pkgPath)) {
                const uninstalledPkg = pkgFullNames.splice(i, 1)[0];
                uninstalledPkgs.push(uninstalledPkg);
                const deps = storeJson.dependencies[uninstalledPkg] || [];
                delete storeJson.dependencies[uninstalledPkg];
                delete storeJson.dependents[uninstalledPkg];
                deps.forEach((dep) => removeDependency(dep, uninstalledPkg, storeJson));
                Array.prototype.push.apply(uninstalledPkgs, tryUninstall(deps, storeJson, pkgPath));
                numberOfUninstalls++;
                continue;
            }
            i++;
        }
    } while (numberOfUninstalls);
    return uninstalledPkgs;
}
function removeDependency(dependentPkgName, uninstalledPkg, storeJson) {
    if (!storeJson.dependents[dependentPkgName])
        return;
    storeJson.dependents[dependentPkgName].splice(storeJson.dependents[dependentPkgName].indexOf(uninstalledPkg), 1);
    if (!storeJson.dependents[dependentPkgName].length) {
        delete storeJson.dependents[dependentPkgName];
    }
}
function removeBins(uninstalledPkg, store, root) {
    const uninstalledPkgJson = requireJson_1.default(path.join(store, uninstalledPkg, '_/package.json'));
    const bins = binify_1.default(uninstalledPkgJson);
    Object.keys(bins).forEach(bin => cbRimraf.sync(path.join(root, 'node_modules/.bin', bin)));
}
function removePkgFromStore(pkgFullName, store) {
    return rimraf(path.join(store, pkgFullName));
}
function rimraf(filePath) {
    return new Promise((resolve, reject) => {
        cbRimraf(filePath, (err) => err ? reject(err) : resolve());
    });
}
//# sourceMappingURL=uninstall.js.map