"use strict";
const debug_1 = require('./debug');
const debug = debug_1.default('pnpm:fetch');
const crypto = require('crypto');
const gunzip = require('gunzip-maybe');
const tar = require('tar-fs');
const fs = require('fs');
/*
 * Fetches a tarball `tarball` and extracts it into `dir`
 */
function fetch(dir, dist, opts) {
    if (dist.location === 'remote') {
        return opts.got.getStream(dist.tarball)
            .then((stream) => fetchStream(dir, dist, opts.log, stream));
    }
    return unpackStream(fs.createReadStream(dist.tarball), dir);
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = fetch;
function fetchStream(dir, dist, log, stream) {
    return new Promise((resolve, reject) => {
        const actualShasum = crypto.createHash('sha1');
        let size = 0;
        let downloaded = 0;
        unpackStream(stream
            .on('response', start)
            .on('data', (_) => { actualShasum.update(_); })
            .on('error', reject), dir).then(finish);
        function start(res) {
            if (res.statusCode !== 200) {
                return reject(new Error('' + dist.tarball + ': invalid response ' + res.statusCode));
            }
            log('download-start');
            if ('content-length' in res.headers) {
                size = +res.headers['content-length'];
                res.on('data', (chunk) => {
                    downloaded += chunk.length;
                    log('downloading', { done: downloaded, total: size });
                });
            }
        }
        function finish() {
            const digest = actualShasum.digest('hex');
            debug(`finish ${dist.shasum} ${dist.tarball}`);
            if (dist.shasum && digest !== dist.shasum) {
                return reject(new Error('' + dist.tarball + ': incorrect shasum (expected ' + dist.shasum + ', got ' + digest + ')'));
            }
            return resolve(dir);
        }
    });
}
function unpackStream(stream, dir) {
    return new Promise((resolve, reject) => {
        stream
            .pipe(gunzip()).on('error', reject)
            .pipe(tar.extract(dir, { strip: 1 })).on('error', reject)
            .on('finish', resolve);
    });
}
//# sourceMappingURL=fetch.js.map