"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const install_1 = require('./install');
/*
 * Install multiple modules into `modules`.
 *
 *     ctx = { }
 *     installMultiple(ctx, { minimatch: '^2.0.0' }, {chokidar: '^1.6.0'}, './node_modules')
 */
function installMultiple(ctx, requiredPkgsMap, optionalPkgsMap, modules, options) {
    requiredPkgsMap = requiredPkgsMap || {};
    optionalPkgsMap = optionalPkgsMap || {};
    const optionalPkgs = Object.keys(optionalPkgsMap)
        .map(pkgName => pkgMeta(pkgName, optionalPkgsMap[pkgName], true));
    const requiredPkgs = Object.keys(requiredPkgsMap)
        .filter(pkgName => !optionalPkgsMap[pkgName])
        .map(pkgName => pkgMeta(pkgName, requiredPkgsMap[pkgName], false));
    ctx.storeJson.dependents = ctx.storeJson.dependents || {};
    ctx.storeJson.dependencies = ctx.storeJson.dependencies || {};
    return Promise.all(optionalPkgs.concat(requiredPkgs).map(function (pkg) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const dependency = yield install_1.default(ctx, pkg, modules, options);
                const depFullName = dependency.fullname;
                ctx.storeJson.dependents[depFullName] = ctx.storeJson.dependents[depFullName] || [];
                if (ctx.storeJson.dependents[depFullName].indexOf(options.dependent) === -1) {
                    ctx.storeJson.dependents[depFullName].push(options.dependent);
                }
                ctx.storeJson.dependencies[options.dependent] = ctx.storeJson.dependencies[options.dependent] || [];
                if (ctx.storeJson.dependencies[options.dependent].indexOf(depFullName) === -1) {
                    ctx.storeJson.dependencies[options.dependent].push(depFullName);
                }
                return dependency;
            }
            catch (err) {
                if (pkg.optional) {
                    console.log('Skipping failed optional dependency ' + pkg.rawSpec + ':');
                    console.log(err.message || err);
                    return null; // is it OK to return null?
                }
                throw err;
            }
        });
    }));
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = installMultiple;
function pkgMeta(name, version, optional) {
    return {
        rawSpec: version === '*' ? name : `${name}@${version}`,
        optional
    };
}
//# sourceMappingURL=installMultiple.js.map