"use strict";
const debug_1 = require('../debug');
const debug = debug_1.default('pnpm:http');
const throat = require('throat');
const got = require('got');
const http_1 = require('http');
const https_1 = require('https');
const caw = require('caw');
const getAuthToken = require('registry-auth-token');
const getRetrier_1 = require('./getRetrier');
const defaults_1 = require('../defaults');
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = (opts) => {
    opts = opts || {};
    const concurrency = +opts.concurrency || 16;
    const forcedRequestOptions = {
        // no worries, the built-in got retries is not used
        // the retry package is used for that purpose
        retries: 0
    };
    const sharedOpts = opts;
    const retrier = getRetrier_1.default({
        retries: opts.fetchRetries || defaults_1.default.fetchRetries,
        factor: opts.fetchRetryFactor || defaults_1.default.fetchRetryFactor,
        minTimeout: opts.fetchRetryMintimeout || defaults_1.default.fetchRetryMintimeout,
        maxTimeout: opts.fetchRetryMaxtimeout || defaults_1.default.fetchRetryMaxtimeout
    });
    const cache = {};
    function getThroater() {
        return throat(concurrency);
    }
    const httpKeepaliveAgent = new http_1.Agent({
        keepAlive: true,
        keepAliveMsecs: 30000
    });
    const httpsKeepaliveAgent = new https_1.Agent({
        keepAlive: true,
        keepAliveMsecs: 30000
    });
    /*
     * waits in line
     */
    const get = retrier((url, options) => {
        const throater = getThroater();
        const key = JSON.stringify([url, options]);
        if (!cache[key]) {
            cache[key] = throater(() => {
                debug(url);
                return got(url, extend(url, options || sharedOpts));
            });
        }
        return cache[key];
    });
    function getJSON(url) {
        return get(url)
            .then((res) => {
            const body = JSON.parse(res.body);
            return body;
        });
    }
    /*
     * like require('got').stream, but throated
     */
    const getStream = retrier((url, options) => {
        const throater = getThroater();
        return new Promise((resolve, reject) => {
            throater(() => {
                debug(url, '[stream]');
                const stream = got.stream(url, extend(url, options || sharedOpts));
                resolve(stream);
                return waiter(stream);
            })
                .catch(reject);
        });
    });
    function waiter(stream) {
        return new Promise((resolve, reject) => {
            stream
                .on('end', resolve)
                .on('error', reject);
        });
    }
    /*
     * Extends `got` options with User Agent headers and stuff
     */
    function extend(url, options) {
        if (!options)
            options = Object.assign({}, forcedRequestOptions);
        if (url.indexOf('https://') === 0) {
            options.agent = caw({ protocol: 'https' }) || httpsKeepaliveAgent;
            const authToken = getAuthToken(url, { recursive: true });
            if (authToken) {
                options.headers = Object.assign({}, options.headers, {
                    authorization: authToken.type + ' ' + authToken.token
                });
            }
        }
        else {
            options.agent = caw({ protocol: 'http' }) || httpKeepaliveAgent;
        }
        return Object.assign({}, options, forcedRequestOptions, {
            headers: Object.assign({}, options.headers, {
                'user-agent': 'https://github.com/rstacruz/pnpm'
            })
        });
    }
    return {
        get: get,
        getJSON: getJSON,
        getStream: getStream
    };
};
//# sourceMappingURL=got.js.map