"use strict";
const npm_1 = require('./npm');
const tarball_1 = require('./tarball');
const github_1 = require('./github');
const local_1 = require('./local');
/**
 * Resolves a package in the NPM registry. Done as part of `install()`.
 *
 *     var npa = require('npm-package-arg')
 *     resolve(npa('rimraf@2'))
 *       .then((res) => {
 *         res.fullname == 'rimraf@2.5.1'
 *         res.dist == {
 *           shasum: '0a1b2c...'
 *           tarball: 'http://...'
 *         }
 *       })
 */
function default_1(spec, opts) {
    switch (spec.type) {
        case 'range':
        case 'version':
        case 'tag':
            return npm_1.default(spec, opts);
        case 'remote':
            return tarball_1.default(spec);
        case 'hosted':
            if (spec.hosted.type === 'github') {
                return github_1.default(spec, opts);
            }
        case 'local':
            return local_1.default(spec, opts);
        default:
            throw new Error(`${spec.rawSpec}: ${spec.type} packages not supported`);
    }
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = default_1;
//# sourceMappingURL=index.js.map