"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const path_1 = require('path');
const spawn = require('cross-spawn');
const pkgFullName_1 = require('../pkgFullName');
const getTarballName_1 = require('./getTarballName');
const requireJson_1 = require('../fs/requireJson');
/**
 * Resolves a package hosted on the local filesystem
 */
function resolveLocal(spec, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const dependencyPath = path_1.resolve(opts.root, spec.spec);
        if (dependencyPath.slice(-4) === '.tgz' || dependencyPath.slice(-7) === '.tar.gz') {
            const name = getTarballName_1.default(dependencyPath);
            return {
                name,
                fullname: getFullName(name, dependencyPath),
                dist: {
                    location: 'local',
                    tarball: dependencyPath
                }
            };
        }
        if (opts.linkLocal) {
            const localPkg = requireJson_1.default(path_1.resolve(dependencyPath, 'package.json'));
            return {
                fullname: getFullName(localPkg.name, dependencyPath),
                dist: {
                    location: 'dir',
                    tarball: dependencyPath,
                }
            };
        }
        return resolveFolder(dependencyPath);
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = resolveLocal;
function resolveFolder(dependencyPath) {
    return new Promise((resolve, reject) => {
        const proc = spawn('npm', ['pack'], {
            cwd: dependencyPath
        });
        let stdout = '';
        proc.stdout.on('data', (data) => {
            stdout += data.toString();
        });
        proc.on('error', reject);
        proc.on('close', (code) => {
            if (code > 0)
                return reject(new Error('Exit code ' + code));
            const tgzFilename = stdout.trim();
            return resolve(tgzFilename);
        });
    })
        .then(tgzFilename => {
        const localPkg = requireJson_1.default(path_1.resolve(dependencyPath, 'package.json'));
        return {
            fullname: getFullName(localPkg.name, dependencyPath),
            dist: {
                location: 'dir',
                tarball: path_1.resolve(dependencyPath, tgzFilename)
            }
        };
    });
}
function getFullName(name, dependencyPath) {
    return pkgFullName_1.default({
        name,
        version: [
            'file',
            removeLeadingSlash(dependencyPath)
        ].join(pkgFullName_1.delimiter)
    });
}
function removeLeadingSlash(pkgPath) {
    return pkgPath.replace(/^[/\\]/, '');
}
//# sourceMappingURL=local.js.map